<?php
/**
 * CRED_Loader
 * 
 * This class is responsible for including all files and getting instances of all objects
 *
 */
class CRED_Loader
{
    private static $prefix='CRED_';
    
    private static function getClassNameFromFileName($file_name)
    {
        if($file_name=='')
        {
            return false;
        }
        
        $name_parts = explode('-',$file_name);
        
        for($i=0; $i<count($name_parts); $i++)
        {
            $name_parts[$i] = ucfirst($name_parts[$i]);
        }
        
        return implode('',$name_parts);
    }
    
    private static function includeFile($file_name)
    {
        if(!file_exists($file_name))
        {
            printf(__('File "%s" doesn\'t exist!','wp-cred'), $file_name);
            return false;
        }
        include_once($file_name);
    }

    public static function includePluginFile($file_name)
    {
        $file_name = CRED_PLUGIN_PATH.'/'.$file_name;
        self::includeFile($file_name);
    }

    
    public static function importClass($file)
    {
        self::includeFile(CRED_CLASSES_DIR.'/'.$file.'.php');    
    }

    public static function includeView($view_name)
    {
        $view_path = CRED_VIEWS_DIR.'/'.$view_name.'.php';
        self::includeFile($view_path);
    }
    
    public static function renderTemplate($___template_name, array $___args=array(), $___template_dir='', $___once=false)
    {
        $___template_dir_path = ($___template_dir <> '') ? $___template_dir.'/' : '';
        $___template_path = CRED_TEMPLATES_DIR.'/'.$___template_dir_path.$___template_name.'.tpl.php';
        if (!file_exists($___template_path))
        {
             printf(__('File "%s" doesn\'t exist!','wp-cred'), $___template_path);
            return false;
        }
        
        if ($___once)
        {
            ob_start();
                extract($___args);
                include_once($___template_path);
            return ob_get_clean();
        }
        ob_start();
            extract($___args);
            include($___template_path);
        return ob_get_clean();
    }
    
    public static function getHelpSettings()
    {
        /*$file_handle = fopen(CRED_HELP_DIR."/cred-help.ini", "rb");

        while (!feof($file_handle) ) 
        {
            $line_of_text = fgets($file_handle);
            $parts = array_map('trim',explode($sep, $line_of_text));
            if (
                (isset($parts[0]) && $parts[0]!='')
                    &&
                (isset($parts[1]) && $parts[1]!='')
            )
                $help_settings_array[$parts[0]]=$parts[1];
        }
        fclose($file_handle);*/
        include CRED_HELP_DIR."/cred-help.ini.php";
        $vars=get_defined_vars();
        // return first defined var (only one)
        return reset($vars);
    }
 
    public static function includeAdminTop($disable_menus = false)
    {
        $disable_menus_css = '<style type="text/css">   #adminmenuwrap, #screen-meta-links {display:none;}   </style>';
        
        self::includeFile(ABSPATH . 'wp-admin/admin-header.php');
        
        if($disable_menus)
        {
            echo $disable_menus_css;
        }
    }

    
/*    public static function includeAdminNotifier()
    {
        self::includeFile(CRED_CLASSES_DIR.'/cred-admin-notifier.php');
    }

*/
    public static function includeRouter()
    {
        self::importClass('cred-router');
    }

    public static function includeWPListTable()
    {
        if(!class_exists('WP_List_Table'))
        {
            $file_name = ABSPATH.'/wp-admin/includes/class-wp-list-table.php';
            self::includeFile($file_name);
        }
    }

    public static function includeListTable($table_name)
    {
        self::includeFile(CRED_TABLES_DIR.'/'.$table_name.'.php');
    }
    
    public static function getListTable($table_name)
    {
       $class_name = self::getClassNameFromFileName($table_name);
       $class_name = self::$prefix.$class_name.'_List_Table';
       $table_name = 'cred-'.$table_name.'-list-table';
       
       if (!class_exists($class_name))
       {
           self::includeWPListTable();
           self::includeListTable($table_name);
       }
       return new $class_name();
    }
    
    public static function getController($short_file_name)
    {   
        $controller_class_name = self::$prefix.'Controller_'.self::getClassNameFromFileName($short_file_name); 

        if(!$controller_class_name)
        {
            return false;
        }
        
        $controller_file_name = 'cred-controller-'.$short_file_name;
        $controller_path = 'controllers/'.$controller_file_name.'.php';
        
        if(!class_exists($controller_class_name))
        {
            self::includePluginFile('controllers/cred-controller.php');
            self::includePluginFile($controller_path);
        }
        if(!class_exists($controller_class_name))
        {
            return false;
        }
        return /*$controller_class_name::getInstance();*/ call_user_func(array($controller_class_name, 'getInstance'));
    }
    
    public static function getModel($short_file_name)
    {
        $model_class_name = self::$prefix.'Model_'.self::getClassNameFromFileName($short_file_name); 

        if(!$model_class_name)
        {
            return false;
        }
        
        $model_file_name = 'cred-model-'.$short_file_name;
        $model_path = CRED_MODELS_DIR.'/'.$model_file_name.'.php';
        
        if(!class_exists($model_class_name))
        {
            self::includeFile(CRED_MODELS_DIR.'/cred-model.php');
            self::includeFile($model_path);
        }
        if(!class_exists($model_class_name))
        {
            return false;
        }
        return /*$model_class_name::getInstance();*/call_user_func(array($model_class_name, 'getInstance'));
    }
}
?>